/*

    MIT License

    Copyright (c) 2017 Motas Electronics Limited (J M HAYES)
    www.motas-synth.uk


    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and any associated documentation files
    (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    Any person wishing to distribute modifications to the Software is
    asked to send the modifications to the original developer so that
    they can be incorporated into the canonical version.  This is,
    however, not a binding provision of this license.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
    ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
    CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
    WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *
 * */

#include "motasfirmware.h"

#include <QMessageBox>
#include <QDebug>

#include <QFile>
#include <QFileDialog>

#include <QTimer>
#include "QPainter"
#include <QRect>
#include <QVector>
#include <QProcess>
#include "crc.h"
#include <QThread>

#include "string.h"
#include "readscreen.h"


MotasFirmware::MotasFirmware(MotasUtility *mainWindow)
{
    this->motasUtility = mainWindow;
}

quint32 MotasFirmware::prepareSysExPacketsFromUnpacked(quint8* outBuf, quint8* buffer, quint32 totalDataSize, int type,
                                                       quint8 msg0, quint8 msg1, quint8 msg2, quint8 model, quint8 channel )
{
    #define MAX_PACKET_LENGTH 64
    quint32 bytesProcessed = 0;
    quint8  outBuffer[MAX_PACKET_LENGTH  + ((MAX_PACKET_LENGTH  + 6) / 7)];

    quint32 bytesSent = 0;
    quint8 startStopContinue = MIDI_SYSEX_START_TRANSFER;

    quint32 bufferSize = totalDataSize;

    // calculate CRC32

    if (totalDataSize % 4 != 0)
    {
         QMessageBox::warning(motasUtility, "Error", "Error in transfer - must be divisible by 4");
         return 0;
    }

    crcInit();
    quint32 crc32 = crcFastSTMF4(buffer, totalDataSize); // calculate CRC of all data


    qDebug() << QString("crc: %1").arg(crc32, 0, 16) << "over total of : " << totalDataSize << "bytes";
    totalDataSize += 4;// make room for CRC32;

    do
    {
        quint32 k = 0;
        quint32 packetLength = totalDataSize - bytesProcessed;
        if (packetLength > MAX_PACKET_LENGTH)
            packetLength = MAX_PACKET_LENGTH;
        quint32 transferSize = packetLength  + ((packetLength  + 6) / 7);

        // convert to data suitable for MIDI;
        while (k < transferSize) // there is an MSB byte to be added for every 7 bytes, since cannot transmit values > 0x7F
        {
            quint8 msb = 0;
            int j;
            for (j = 0; j < 7; j++)
            {
                quint32 offset = bytesProcessed + j;
                if (offset >= totalDataSize)
                    break;
                if (offset < bufferSize)
                {
                    if (buffer[offset] > 0x7F)
                        msb |= (0x40 >> j);
                } else
                {
                    if (((crc32 >> ((offset - bufferSize) * 8)) & 0xFF) > 0x7F)
                        msb |= (0x40 >> j);
                }

            }
            outBuffer[k++] = msb;
            for (j = 0; j < 7; j++)
            {
                if (bytesProcessed >= totalDataSize)
                    break;
                if (k >= transferSize)
                    break;

                if ((bytesProcessed) < bufferSize)
                {
                    outBuffer[k++] =  buffer[bytesProcessed] & 0x7F;
                } else
                {
                    outBuffer[k++] =  (crc32 >> (bytesProcessed - bufferSize) * 8) & 0x7F;
                }
                bytesProcessed++;
            }
        }
        if (bytesProcessed >= totalDataSize)
        {
            startStopContinue = MIDI_SYSEX_END_TRANSFER;
            if (totalDataSize < MAX_PACKET_LENGTH)
            {
                startStopContinue = MIDI_SYSEX_START_AND_END_TRANSFER;
                qDebug() << "Sending single packet";
            }
        }
        else if (bytesProcessed > packetLength)
            startStopContinue = MIDI_SYSEX_CONTINUE_TRANSFER;
        bytesSent += Utilities::lowLevelCreateMIDIStream(outBuffer, transferSize, type, startStopContinue, msg0, msg1, msg2,  &outBuf[bytesSent],
                                                         model, channel);
    } while (bytesProcessed < totalDataSize);

    return bytesSent;
}
