/*

    Copyright (c) 2017 Motas Electronics Limited (J M HAYES)
    www.motas-synth.uk


    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and any associated documentation files
    (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    Any person wishing to distribute modifications to the Software is
    asked to send the modifications to the original developer so that
    they can be incorporated into the canonical version.  This is,
    however, not a binding provision of this license.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
    ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
    CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
    WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *
 * */

#include "screencapture.h"
#include "ui_screencapture.h"
#include "QDebug"
#include "QResizeEvent"

ScreenCapture::ScreenCapture(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::ScreenCapture)
{
    ui->setupUi(this);
    smoothImage = false;
    active = false;




}

ScreenCapture::~ScreenCapture()
{
    qDebug() << "Screencapture closed";
    delete ui;
}

void ScreenCapture::setImage(QPixmap p, bool saveImage, QString fileLoc)
{
    this->pixmap = p;
    if (!active)
    {
        ui->labelImage->clear();
        this->resize(512,256);
        if (this->size().width() == 512) // wait until the resize has completed
        {
            active = true;
        }
    } else
    {
        showPixmap(ui->labelImage->width(), false);


        if (saveImage)
        {
            QString fileName = fileLoc;
            static quint32 i = 0;

            static QString oldLoc = "";
            if (oldLoc != fileLoc)
            {
                i = 0;
                oldLoc = fileLoc;
            }


            fileName += "_" + QString::number(i++) + ".png";

            QFile file(fileName);
            file.open(QIODevice::WriteOnly);

            p.save(&file, "PNG");
            file.close();
        }
        //qDebug() << fileName;
        //notSaving = true;


    }






}
void ScreenCapture::setSmoothImage(bool smooth)
{
    smoothImage = smooth;
}

void ScreenCapture::showPixmap(int w, bool override)
{
    if ((override || active) && !pixmap.isNull())
    {

       // qDebug() << "set pixmap";
        if (smoothImage)
            ui->labelImage->setPixmap(pixmap.scaledToWidth(w,  Qt::SmoothTransformation));
        else
            ui->labelImage->setPixmap(pixmap.scaledToWidth(w,  Qt::FastTransformation));
    }

}


void ScreenCapture::resetWindow()
{
     showPixmap(512, false);
  //   ui->labelImage->clear();
  //   ui->labelImage->setGeometry(0,0,512,256);
  //   this->setGeometry(0,0,512,256);
}

void ScreenCapture::resizeEvent(QResizeEvent * resize)
{

    if (!pixmap.isNull() && resize != NULL)
    {
        //qDebug() << "resize event width: " << resize->size().width();
        int w = resize->size().width();
        showPixmap(w, false);
      //  int h = pixmap.height();

    }
}

void ScreenCapture::closeEvent(QCloseEvent *)
{
    qDebug() << "window closed";
    active = false;
    ui->labelImage->clear();
    this->resize(512,256);

}

